@@ -3,7 +3,7 @@
 # IO::Socket::SSL:
 # provide an interface to SSL connections similar to IO::Socket modules
 #
-# Current Code Shepherd: Steffen Ullrich <steffen at genua.de>
+# Current Code Shepherd: Steffen Ullrich <sullr at cpan.org>
 # Code Shepherd before: Peter Behroozi, <behrooz at fas.harvard.edu>
 #
 # The original version of this module was written by
@@ -20,7 +20,7 @@ use Errno qw( EAGAIN ETIMEDOUT );
 use Carp;
 use strict;
 
-our $VERSION = 1.87;
+our $VERSION = '1.93';
 
 use constant SSL_VERIFY_NONE => Net::SSLeay::VERIFY_NONE();
 use constant SSL_VERIFY_PEER => Net::SSLeay::VERIFY_PEER();
@@ -85,7 +85,7 @@ use vars qw(@ISA $SSL_ERROR @EXPORT );
 my @caller_force_inet4; # in case inet4 gets forced we store here who forced it
 
 BEGIN {
-    # declare @ISA depeding of the installed socket class
+    # declare @ISA depending of the installed socket class
 
     # try to load inet_pton from Socket or Socket6
     local $SIG{__DIE__}; local $SIG{__WARN__}; # be silent
@@ -103,9 +103,9 @@ BEGIN {
     # try IO::Socket::IP or IO::Socket::INET6 for IPv6 support
     if ( $ip6 ) {
 
-	# if we have IO::Socket::IP >= 0.11 we will use this in preference
+	# if we have IO::Socket::IP >= 0.20 we will use this in preference
 	# because it can handle both IPv4 and IPv6
-	if ( eval { require IO::Socket::IP; IO::Socket::IP->VERSION(0.11); } ) {
+	if ( eval { require IO::Socket::IP; IO::Socket::IP->VERSION(0.20); } ) {
 	    @ISA = qw(IO::Socket::IP);
 	    constant->import( CAN_IPV6 => "IO::Socket::IP" );
 
@@ -134,6 +134,7 @@ BEGIN {
     # Do Net::SSLeay initialization
     Net::SSLeay::load_error_strings();
     Net::SSLeay::SSLeay_add_ssl_algorithms();
+    Net::SSLeay::OpenSSL_add_all_digests();
     Net::SSLeay::randomize();
 }
 
@@ -187,8 +188,8 @@ BEGIN {
 }
 
 # Export some stuff
-# inet4|inet6|debug will be handeled by myself, everything
-# else will be handeld the Exporter way
+# inet4|inet6|debug will be handled by myself, everything
+# else will be handled the Exporter way
 sub import {
     my $class = shift;
 
@@ -253,7 +254,7 @@ sub configure {
 
     # force initial blocking
     # otherwise IO::Socket::SSL->new might return undef if the
-    # socket is nonblocking and it fails to connect immediatly
+    # socket is nonblocking and it fails to connect immediately
     # for real nonblocking behavior one should create a nonblocking
     # socket and later call connect explicitly
     my $blocking = delete $arg_hash->{Blocking};
@@ -334,9 +335,13 @@ sub configure_SSL {
     {
 	my $use_default = 1;
 	for (qw( SSL_cert SSL_cert_file SSL_key SSL_key_file SSL_ca_file SSL_ca_path )) {
-	    next if ! exists $arg_hash->{$_};
+	    next if ! defined $arg_hash->{$_};
+	    # some apps set keys '' to signal that it is not set, replace with undef
+	    if ( $arg_hash->{$_} eq '' ) {
+		$arg_hash->{$_} = undef;
+		next;
+	    }
 	    $use_default = 0;
-	    last;
 	}
 	if ( $use_default ) {
 	    my %ca = 
@@ -795,11 +800,7 @@ sub sysread {
 
 sub peek {
     my $self = shift;
-    if (Net::SSLeay::OPENSSL_VERSION_NUMBER() >= 0x0090601f) {
-	return $self->generic_read(\&Net::SSLeay::peek, @_);
-    } else {
-	return $self->error("SSL_peek not supported for OpenSSL < v0.9.6a");
-    }
+    return $self->generic_read(\&Net::SSLeay::peek, @_);
 }
 
 
@@ -918,26 +919,7 @@ sub readline {
 
     my ($delim0,$delim1) = $/ eq '' ? ("\n\n","\n"):($/,'');
 
-    if ( Net::SSLeay::OPENSSL_VERSION_NUMBER() < 0x0090601f ) {
-	# no usable peek - need to read byte after byte
-	die "empty \$/ is not supported if I don't have peek" if $delim1 ne '';
-	my $buf = '';
-	while (1) {
-	    my $rv = $self->sysread($buf,1,length($buf));
-	    if ( ! defined $rv ) {
-		next if $!{EINTR};                     # retry
-		last if $!{EAGAIN} || $!{EWOULDBLOCK}; # use everything so far
-		return;                                # return error
-	    } elsif ( ! $rv ) {
-		last
-	    }
-	    index($buf,$delim0) >= 0 and last;
-	}
-	return $buf;
-    }
-
-
-    # find first occurence of $delim0 followed by as much as possible $delim1
+    # find first occurrence of $delim0 followed by as much as possible $delim1
     my $buf = '';
     my $eod = 0;  # pointer into $buf after $delim0 $delim1*
     my $ssl = $self->_get_ssl_object or return;
@@ -1310,7 +1292,7 @@ sub dump_peer_certificate {
 	    $ipn = inet_pton(AF_INET6,$identity) 
 		or croak "'$identity' is not IPv6, but neither IPv4 nor hostname";
 	} elsif ( $identity =~m{^\d+\.\d+\.\d+\.\d+$} ) {
-	     # definitly no hostname, try IPv4
+	     # definitely no hostname, try IPv4
 	    $ipn = inet_aton( $identity ) or croak "'$identity' is not IPv4, but neither IPv6 nor hostname";
 	} else {
 	    # assume hostname, check for umlauts etc
@@ -1346,7 +1328,7 @@ sub dump_peer_certificate {
 	while (@altNames) {
 	    my ($type, $name) = splice (@altNames, 0, 2);
 	    if ( $ipn and $type == GEN_IPADD ) {
-		# exakt match needed for IP
+		# exact match needed for IP
 		# $name is already packed format (inet_xton)
 		return 1 if $ipn eq $name;
 
@@ -1425,7 +1407,7 @@ sub error {
 	push @err, Net::SSLeay::ERR_error_string($err);
 	$DEBUG>=2 && DEBUG( $error."\n".$self->get_ssleay_error());
     }
-    # if no new error occured report last again
+    # if no new error occurred report last again
     if ( ! @err and my $err = 
 	ref($self) ? ${*$self}{'_SSL_last_err'} : $SSL_ERROR ) {
 	push @err,$err;
@@ -1634,7 +1616,7 @@ sub new {
 	Net::SSLeay::CTX_set_options($ctx, 0x00400000);
     }
 
-    # if we don't set session_id_context if client certicate is expected
+    # if we don't set session_id_context if client certificate is expected
     # client session caching will fail
     # if user does not provide explicit id just use the stringification
     # of the context
@@ -1657,7 +1639,7 @@ sub new {
 	    # on server side SSL_npn_protocols means a list of advertised protocols
 	    Net::SSLeay::CTX_set_next_protos_advertised_cb($ctx, $proto_list);
 	} else {
-	    # on client side SSL_npn_protocols means a list of prefered protocols
+	    # on client side SSL_npn_protocols means a list of preferred protocols
 	    # negotiation algorithm used is "as-openssl-implements-it"
 	    Net::SSLeay::CTX_set_next_proto_select_cb($ctx, $proto_list);
 	}
@@ -1672,22 +1654,18 @@ sub new {
     }
 
     if ($arg_hash->{'SSL_check_crl'}) {
-	if (Net::SSLeay::OPENSSL_VERSION_NUMBER() >= 0x0090702f) {
-	    Net::SSLeay::X509_STORE_set_flags(
-		Net::SSLeay::CTX_get_cert_store($ctx),
-		Net::SSLeay::X509_V_FLAG_CRL_CHECK()
-	    );
-	    if ($arg_hash->{'SSL_crl_file'}) {
-		my $bio = Net::SSLeay::BIO_new_file($arg_hash->{'SSL_crl_file'}, 'r');
-		my $crl = Net::SSLeay::PEM_read_bio_X509_CRL($bio);
-		if ( $crl ) {
-		    Net::SSLeay::X509_STORE_add_crl(Net::SSLeay::CTX_get_cert_store($ctx), $crl);
-		} else {
-		    return IO::Socket::SSL->error("Invalid certificate revocation list");
-		}
+	Net::SSLeay::X509_STORE_set_flags(
+	    Net::SSLeay::CTX_get_cert_store($ctx),
+	    Net::SSLeay::X509_V_FLAG_CRL_CHECK()
+	);
+	if ($arg_hash->{'SSL_crl_file'}) {
+	    my $bio = Net::SSLeay::BIO_new_file($arg_hash->{'SSL_crl_file'}, 'r');
+	    my $crl = Net::SSLeay::PEM_read_bio_X509_CRL($bio);
+	    if ( $crl ) {
+		Net::SSLeay::X509_STORE_add_crl(Net::SSLeay::CTX_get_cert_store($ctx), $crl);
+	    } else {
+		return IO::Socket::SSL->error("Invalid certificate revocation list");
 	    }
-	} else {
-	    return IO::Socket::SSL->error("CRL not supported for OpenSSL < v0.9.7b");
 	}
     }
 
@@ -1745,7 +1723,7 @@ sub new {
 	}
 
 	if ( keys %sni > 1 or ! exists $sni{''} ) {
-	    # we definitly want SNI support
+	    # we definitely want SNI support
 	    $can_server_sni or return IO::Socket::SSL->error(
 		"Server side SNI not supported for this openssl/Net::SSLeay");
 	    $_ = $_->{ctx} for( values %sni);
@@ -1915,6 +1893,7 @@ sub DESTROY {
 
 1;
 
+__END__
 
 =head1 NAME
 
@@ -1937,7 +1916,7 @@ IO::Socket::SSL -- SSL sockets with IO::Socket interface
 	# on OpenBSD instead: SSL_ca_file => '/etc/ssl/cert.pem'
 
 	# easy hostname verification 
-	SSL_verifycb_name => 'foo.bar', # defaults to PeerHost
+	SSL_verifycn_name => 'foo.bar', # defaults to PeerHost
 	SSL_verifycn_schema => 'http',
 
 	# SNI support
@@ -1984,8 +1963,6 @@ IO::Socket::SSL -- SSL sockets with IO::Socket interface
     print $sock .... 
 
 
-
-
 =head1 DESCRIPTION
 
 This module provides an interface to SSL sockets, similar to other IO::Socket
@@ -1993,7 +1970,6 @@ modules. Because of that, it can be used to make existing programs using
 IO::Socket::INET or similar modules to provide SSL encryption without much
 effort.
 IO::Socket::SSL supports all the extra features that one needs to write a
-
 full-featured SSL client or server application: multiple SSL contexts, cipher
 selection, certificate verification, Server Name Indication (SNI), Next
 Protocol Negotiation (NPN), SSL version selection and more.
@@ -2012,8 +1988,8 @@ The choice of the super class depends on the installed modules:
 
 =item *
 
-If IO::Socket::IP is installed it will use this module as super class,
-transparently providing IPv6 and IPv4 support.
+If IO::Socket::IP with at least version 0.20 is installed it will use this
+module as super class, transparently providing IPv6 and IPv4 support.
 
 =item *
 
@@ -2053,7 +2029,7 @@ that came bundled with IO::Socket::INET, plus (optionally) the ones that follow:
 
 =item SSL_hostname
 
-This can be given to specifiy the hostname used for SNI, which is needed if you
+This can be given to specify the hostname used for SNI, which is needed if you
 have multiple SSL hostnames on the same IP address. If not given it will try to
 determine hostname from PeerAddr, which will fail if only IP was given or if
 this argument is used within start_SSL.
@@ -2109,7 +2085,7 @@ you are setting up an SSL client.  If this is set to 0 (the default), then you w
 only need a certificate and key if you are setting up a server.
 
 SSL_use_cert will implicitly be set if SSL_server is set.
-For convinience it is also set if it was not given but a cert was given for use
+For convenience it is also set if it was not given but a cert was given for use
 (SSL_cert_file or similar).
 
 =item SSL_server
@@ -2145,7 +2121,7 @@ Examples:
 
 This option can be used instead of C<SSL_cert_file> to specify the certificate.
 
-Instead with a file the certifcate is given as an X509* object or array of
+Instead with a file the certificate is given as an X509* object or array of
 X509* objects, where the first X509* is the internal representation of the
 certificate while the following ones are extra certificates.
 The option is useful if you create your certificate dynamically (like in a SSL
@@ -2198,7 +2174,7 @@ If you want to verify that the peer certificate has been signed by a reputable
 certificate authority, then you should use this option to locate the file
 containing the certificateZ<>(s) of the reputable certificate authorities if it is
 not already in the file F<certs/my-ca.pem>.
-If you definitly want no SSL_ca_file used you should set it to undef.
+If you definitely want no SSL_ca_file used you should set it to undef.
 
 =item SSL_ca_path
 
@@ -2207,7 +2183,7 @@ yourself up a directory containing several trusted certificates as separate file
 as well as an index of the certificates.  If you want to use that directory for
 validation purposes, and that directory is not F<ca/>, then use this option to
 point IO::Socket::SSL to the right place to look.
-If you definitly want no SSL_ca_path used you should set it to undef.
+If you definitely want no SSL_ca_path used you should set it to undef.
 
 =item SSL_verify_mode
 
@@ -2357,7 +2333,7 @@ actual socket connection works but the SSL negotiation fails, as in the case of
 an HTTP client connecting to an HTTPS server.  Passing a subroutine ref attached
 to this parameter allows you to gain control of the orphaned socket instead of having it
 be closed forcibly.	 The subroutine, if called, will be passed two parameters:
-a reference to the socket on which the SSL negotiation failed and and the full
+a reference to the socket on which the SSL negotiation failed and the full
 text of the error message.
 
 =item SSL_npn_protocols
@@ -2398,7 +2374,7 @@ if you have used shutdown() or are working on a copy of a socket.
 =item SSL_fast_shutdown
 
 If set to true only a unidirectional shutdown will be done, e.g. only the
-close_notify (see SSL_shutdown(3)) will be called. Otherwise a bidrectional
+close_notify (see SSL_shutdown(3)) will be called. Otherwise a bidirectional
 shutdown will be done. If used within close() it defaults to true, if used
 within stop_SSL() it defaults to false.
 
@@ -2591,7 +2567,7 @@ This is the opposite of start_SSL(), e.g. it will shutdown the SSL connection
 and return to the class before start_SSL(). It gets the same arguments as close(),
 in fact close() calls stop_SSL() (but without downgrading the class).
 
-Will return true if it suceeded and undef if failed. This might be the case for
+Will return true if it succeeded and undef if failed. This might be the case for
 non-blocking sockets. In this case $! is set to EAGAIN and the ssl error to
 SSL_WANT_READ or SSL_WANT_WRITE. In this case the call should be retried again with
 the same arguments once the socket is ready is until it succeeds.
@@ -2671,7 +2647,7 @@ and read/sysread families instead.
 =head1 ERROR HANDLING
 
 If an SSL specific error occurs the global variable C<$SSL_ERROR> will be set.
-If the error occured on an existing SSL socket the method C<errstr> will
+If the error occurred on an existing SSL socket the method C<errstr> will
 give access to the latest socket specific error.
 Both C<$SSL_ERROR> and C<errstr> method give a dualvar similar to C<$!>, e.g.
 providing an error number in numeric context or an error description in string
@@ -2681,7 +2657,7 @@ context.
 
 If you have a non-blocking socket, the expected behavior on read, write, accept
 or connect is to set C<$!> to EAGAIN if the operation can not be completed
-immediatly.
+immediately.
 
 With SSL there are cases, like with SSL handshakes, where the write operation
 can not be completed until it can read from the socket or vice versa. 
